@@ -34,7 +34,7 @@ class AzureDeployer:
         self._confirm_preliminary()
 
         self._prep_automate_all()
-        self._inspect_project()
+        self._get_azure_settings()
         self.sd._add_simple_deploy_req()
 
         # While only supporting automate-all, this will be called after we
@@ -104,15 +104,6 @@ class AzureDeployer:
             sys.exit()
 
 
-    def _inspect_project(self):
-        """Inspect the project, and pull information needed by multiple steps.
-        """
-        # Get platform-agnostic information about the project.
-        self.sd._inspect_project()
-
-        self._get_azure_settings()
-
-
     def _get_azure_settings(self):
         """Get any azure-specific settings that are already in place.
         """
