@@ -5,6 +5,7 @@ import sys, os, re, subprocess
 from django.conf import settings
 from django.core.management.base import CommandError
 from django.core.management.utils import get_random_secret_key
+from django.utils.crypto import get_random_string
 
 from simple_deploy.management.commands.utils import deploy_messages as d_msgs
 from simple_deploy.management.commands.utils import deploy_messages_heroku as dh_msgs
@@ -27,7 +28,7 @@ class HerokuDeployer:
         self._prep_automate_all()
         self._get_heroku_app_info()
         self._set_heroku_env_var()
-        self._inspect_project()
+        self._get_heroku_settings()
         self.sd._add_simple_deploy_req()
         self._generate_procfile()
         self._add_gunicorn()
@@ -65,7 +66,8 @@ class HerokuDeployer:
 
         if confirmed.lower() in ('y', 'yes'):
             self.sd.write_output("  Running `heroku create`...")
-            output = subprocess.run(['heroku', 'create'], capture_output=True)
+            cmd = 'heroku create'
+            output = self.sd.execute_subp_run(cmd)
             self.sd.write_output(output)
         else:
             # Quit and have the user run the command again; don't assume not
@@ -79,7 +81,8 @@ class HerokuDeployer:
         # We assume the user has already run 'heroku create', or --automate-all
         #   has run it. If it hasn't been run, we'll quit and tell them to do so.
         self.sd.write_output("  Inspecting Heroku app...")
-        apps_info = subprocess.run(["heroku", "apps:info"], capture_output=True)
+        cmd = 'heroku apps:info'
+        apps_info = self.sd.execute_subp_run(cmd)
         self.sd.write_output(apps_info)
 
         # Turn stdout info into a list of strings that we can then parse.
@@ -93,7 +96,8 @@ class HerokuDeployer:
             self.sd.write_output(f"    Found Heroku app: {self.heroku_app_name}")
         else:
             # Let user know they need to run `heroku create`.
-            self.sd.write_output(dh_msgs.no_heroku_app_detected)
+            self.sd.write_output(dh_msgs.no_heroku_app_detected,
+                write_to_console=False)
             raise CommandError(dh_msgs.no_heroku_app_detected)
 
 
@@ -102,23 +106,13 @@ class HerokuDeployer:
         This is mostly used to modify settings for the deployed project.
         """
         self.sd.write_output("  Setting Heroku environment variable...")
-        output = subprocess.run(["heroku", "config:set", "ON_HEROKU=1"],
-                capture_output=True)
+        cmd = 'heroku config:set ON_HEROKU=1'
+        output = self.sd.execute_subp_run(cmd)
         self.sd.write_output(output)
         self.sd.write_output("    Set ON_HEROKU=1.")
         self.sd.write_output("    This is used to define Heroku-specific settings.")
 
 
-    def _inspect_project(self):
-        """Inspect the project, and pull information needed by multiple steps.
-        """
-
-        # Get platform-agnostic information about the project.
-        self.sd._inspect_project()
-
-        self._get_heroku_settings()
-
-
     def _get_heroku_settings(self):
         """Get any heroku-specific settings that are already in place.
         """
@@ -142,16 +136,19 @@ class HerokuDeployer:
         """Create Procfile, if none present."""
 
         #   Procfile should be in project root, if present.
-        self.sd.write_output(f"\n  Looking in {self.sd.project_root} for Procfile...")
-        procfile_present = 'Procfile' in os.listdir(self.sd.project_root)
+        self.sd.write_output(f"\n  Looking in {self.sd.git_path} for Procfile...")
+        procfile_present = 'Procfile' in os.listdir(self.sd.git_path)
 
         if procfile_present:
             self.sd.write_output("    Found existing Procfile.")
         else:
             self.sd.write_output("    No Procfile found. Generating Procfile...")
-            proc_command = f"web: gunicorn {self.sd.project_name}.wsgi --log-file -"
+            if self.sd.nested_project:
+                proc_command = f"web: gunicorn {self.sd.project_name}.{self.sd.project_name}.wsgi --log-file -"
+            else:
+                proc_command = f"web: gunicorn {self.sd.project_name}.wsgi --log-file -"
 
-            with open(f"{self.sd.project_root}/Procfile", 'w') as f:
+            with open(f"{self.sd.git_path}/Procfile", 'w') as f:
                 f.write(proc_command)
 
             self.sd.write_output("    Generated Procfile with following process:")
@@ -189,6 +186,7 @@ class HerokuDeployer:
             # Let user know there's a nonempty ALLOWED_HOSTS, that doesn't 
             #   contain the current Heroku URL.
             msg = d_msgs.allowed_hosts_not_empty_msg(heroku_host)
+            self.write_output(msg, write_to_console=False)
             raise CommandError(msg)
 
 
@@ -303,7 +301,8 @@ class HerokuDeployer:
         # Taken from: https://stackoverflow.com/a/56828137/748891
 
         self.sd.write_output("  Setting DEBUG env var...")
-        output = subprocess.run(["heroku", "config:set", f"DEBUG=FALSE"], capture_output=True)
+        cmd = 'heroku config:set DEBUG=FALSE'
+        output = self.sd.execute_subp_run(cmd)
         self.sd.write_output(output)
         self.sd.write_output("    Set DEBUG config variable to FALSE.")
 
@@ -317,12 +316,17 @@ class HerokuDeployer:
     def _configure_secret_key(self):
         """Use an env var to manage the secret key."""
         # Generate a new key.
-        new_secret_key = get_random_secret_key()
+        if self.sd.on_windows:
+            # Non-alphanumeric keys have been problematic on Windows.
+            new_secret_key = get_random_string(length=50,
+                    allowed_chars='abcdefghijklmnopqrstuvwxyz0123456789')
+        else:
+            new_secret_key = get_random_secret_key()
 
         # Set the new key as an env var on Heroku.
         self.sd.write_output("  Setting new secret key for Heroku...")
-        output = subprocess.run(["heroku", "config:set", f"SECRET_KEY={new_secret_key}"],
-                capture_output=True)
+        cmd = f"heroku config:set SECRET_KEY={new_secret_key}"
+        output = self.sd.execute_subp_run(cmd)
         self.sd.write_output(output)
         self.sd.write_output("    Set SECRET_KEY config variable.")
 
@@ -341,18 +345,24 @@ class HerokuDeployer:
         self.sd.write_output("\n\nCommitting and pushing project...")
 
         self.sd.write_output("  Adding changes...")
-        output = subprocess.run(['git', 'add', '.'], capture_output=True)
+        cmd = 'git add .'
+        output = self.sd.execute_subp_run(cmd)
         self.sd.write_output(output)
         self.sd.write_output("  Committing changes...")
-        output = subprocess.run(['git', 'commit', '-am', '"Configured project for deployment."'],
-                capture_output=True)
+        # output = subprocess.run(['git', 'commit', '-am', '"Configured project for deployment."'],
+        #         capture_output=True)
+        # If we write this command as a string, the commit message will be split
+        #   incorrectly.
+        cmd_parts = ['git', 'commit', '-am', '"Configured project for deployment."']
+        output = self.sd.execute_subp_run_parts(cmd_parts)
         self.sd.write_output(output)
 
         self.sd.write_output("  Pushing to heroku...")
 
         # Get the current branch name. Get the first line of status output,
         #   and keep everything after "On branch ".
-        git_status = subprocess.run(['git', 'status'], capture_output=True)
+        cmd = 'git status'
+        git_status = self.sd.execute_subp_run(cmd)
         self.sd.write_output(git_status)
         status_str = git_status.stdout.decode()
         self.current_branch = status_str.split('\n')[0][10:]
@@ -365,21 +375,24 @@ class HerokuDeployer:
         self.sd.write_output(f"    Pushing branch {self.current_branch}...")
         if self.current_branch in ('main', 'master'):
             cmd = f"git push heroku {self.current_branch}"
-            self.sd.execute_command(cmd)
         else:
             cmd = f"git push heroku {self.current_branch}:main"
-            self.sd.execute_command(cmd)
+        self.sd.execute_command(cmd)
 
         # Run initial set of migrations.
         self.sd.write_output("  Migrating deployed app...")
-        output = subprocess.run(['heroku', 'run', 'python', 'manage.py', 'migrate'],
-                capture_output=True)
+        if self.sd.nested_project:
+            cmd = f"heroku run python {self.sd.project_name}/manage.py migrate"
+        else:
+            cmd = 'heroku run python manage.py migrate'
+        output = self.sd.execute_subp_run(cmd)
+
         self.sd.write_output(output)
 
         # Open Heroku app, so it simply appears in user's browser.
         self.sd.write_output("  Opening deployed app in a new browser tab...")
-        output = subprocess.run(['heroku', 'open'],
-                capture_output=True)
+        cmd = 'heroku open'
+        output = self.sd.execute_subp_run(cmd)
         self.sd.write_output(output)
 
 
